﻿using RabbitMQ.Client;
using RabbitMQ.Client.Events;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

namespace Subscriber
{
    class Program
    {
        static void Main(string[] args)
        {
            var factory = new ConnectionFactory();
            factory.HostName = "localhost";

            using (var con = factory.CreateConnection())
            {
                using (var channel = con.CreateModel())
                {
                    channel.QueueDeclare(queue: "worker",
                                         durable: false,
                                         exclusive: false,
                                         autoDelete: false,
                                         arguments: null);

                    var consumer = new EventingBasicConsumer(channel);
                    consumer.Received += (sender, e) =>
                    {
                        var body = e.Body;
                        var basicProperties = e.BasicProperties;
                        var replyProperties = channel.CreateBasicProperties();
                        replyProperties.CorrelationId = basicProperties.CorrelationId;

                        var url = Encoding.UTF8.GetString(body);
                        Console.WriteLine("Processing url: {0}", url);

                        string content = "";

                        try
                        {
                            content = ProcessUrl(url);
                        }
                        catch (Exception)
                        {
                            Console.WriteLine("failed to read url {0}", url);
                        }
                        finally
                        {
                            var response = Encoding.UTF8.GetBytes(content);
                            channel.BasicPublish(exchange: "", routingKey: basicProperties.ReplyTo, basicProperties: replyProperties, body: response);
                            channel.BasicAck(deliveryTag: e.DeliveryTag, multiple: false);
                        }
                    };

                    channel.BasicConsume(queue: "worker",
                                         autoAck: false,
                                         consumer: consumer);

                    Console.ReadLine();
                }
            }

            Console.ReadLine();
        }

        private static string ProcessUrl(string url)
        {
            string content = "";
            using (WebClient client = new WebClient())
            {
                content = client.DownloadString(url);
            }

            return content;
        }
    }
}
